/*
 * Project Scelight External Module SDK.
 * 
 * Copyright (c) 2013 Andras Belicza <iczaaa@gmail.com>
 * 
 * See SDK-LICENSE.txt.
 */
package hu.belicza.andras.helloworldextmod;

import hu.scelightapi.BaseExtModule;
import hu.scelightapi.IModEnv;
import hu.scelightapi.IServices;
import hu.scelightapi.service.repfoldermonitor.INewRepEvent;
import hu.scelightapi.service.repfoldermonitor.INewRepListener;
import hu.scelightapi.service.sc2monitor.GameChangeAdapter;
import hu.scelightapi.service.sc2monitor.IGameChangeEvent;
import hu.scelightapi.service.sc2monitor.IGameChangeListener;
import hu.scelightapibase.bean.IExtModManifestBean;
import hu.scelightapibase.bean.settings.ISettingsBean;
import hu.scelightapibase.bean.settings.type.IBoolSetting;
import hu.scelightapibase.bean.settings.type.IIntSetting;
import hu.scelightapibase.bean.settings.type.INodeSetting;
import hu.scelightapibase.bean.settings.type.ISetting;
import hu.scelightapibase.bean.settings.type.ISettingsGroup;
import hu.scelightapibase.gui.comp.multipage.IPage;
import hu.scelightapibase.gui.comp.multipage.IPageCompCreator;
import hu.scelightapibase.gui.icon.IRIcon;
import hu.scelightapibase.util.IRHtml;
import hu.scelightapibase.util.ISkillLevel;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.JComponent;

/**
 * Hello World example Scelight external module.
 * 
 * <p>
 * This external module registers an {@link INewRepListener} and an {@link IGameChangeListener} to listen to these events, and gathers them in a list.<br>
 * Also adds a new {@link IPage} to the main multi-page component of the main frame, and in a table it displays events. The tool bar over the table
 * also contains buttons to refresh the table, to remove selected events or to add custom events.
 * </p>
 * 
 * <p>
 * The module also uses settings, one where you can tell if you like the module and another one where you can set a table row number limit.
 * </p>
 * 
 * @author Andras Belicza
 */
public class HelloWorld extends BaseExtModule {
	
	/** Ricon of the Hello World. */
	IRIcon         I_ICON;
	
	/** Cross icon (delete). */
	IRIcon         I_CROSS;
	
	/** A plus button icon (add new row). */
	IRIcon         I_PLUS_BUTTON;
	
	/** An arrow circle button icon (refresh). */
	IRIcon         I_ARROW_CIRCLE_315;
	
	/** A dummy happy icon. */
	IRIcon         I_DUMMY_HAPPY;
	
	/** A dummy sad icon. */
	IRIcon         I_DUMMY_SAD;
	
	/** Hello World test help content. */
	IRHtml         HELLO_WORLD_HELP;
	
	
	/** Our node in the Settings dialog. */
	INodeSetting   NODE_SETTING;
	
	/** Settings group. */
	ISettingsGroup GROUP_HELLO_WORLD;
	
	/** Like setting. */
	IBoolSetting   LIKE_SETTING;
	
	/** Max table rows setting. */
	IIntSetting   MAX_TABLE_ROWS_SETTING;
	
	
	/** Settings of the external module. */
	ISettingsBean settings;
	
	
	/** List of the logged events.<br>
	 * Since multiple threads are accessing it, access must be externally synchronized! */
	final List< EventData > eventDataList = new ArrayList<>();
	
	
	@Override
	public void init( final IExtModManifestBean manifest, final IServices services, final IModEnv modEnv ) {
		// Properly delegate to super first
		super.init( manifest, services, modEnv );
		
		loadResources();
		
		loadSettings();
		
		installRepListener();
		
		installGameChangeListener();
		
		// Build the GUI. As all GUI related things, this has to be called in the EDT!
		guiUtils.runInEDT( new Runnable() {
			@Override
			public void run() {
				buildGui();
			}
		} );
		
		logger.debug( manifest.getName() + " started successfully." );
	}
	
	/**
	 * Loads static resources like icons and HTML templates.
	 */
	private void loadResources() {
		// Create Hello World icon resource.
		I_ICON = guiFactory.newRIcon( getClass().getResource( "icon/hw-icon.png" ) );
		I_CROSS = guiFactory.newRIcon( getClass().getResource( "icon/cross.png" ) );
		I_PLUS_BUTTON = guiFactory.newRIcon( getClass().getResource( "icon/plus-button.png" ) );
		I_ARROW_CIRCLE_315 = guiFactory.newRIcon( getClass().getResource( "icon/arrow-circle-315.png" ) );
		I_DUMMY_HAPPY = guiFactory.newRIcon( getClass().getResource( "icon/dummy-happy.png" ) );
		I_DUMMY_SAD = guiFactory.newRIcon( getClass().getResource( "icon/dummy-sad.png" ) );
		
		HELLO_WORLD_HELP = guiFactory.newRHtml( "Hello World Help", getClass().getResource( "help/hello-world-help.html" ),
				"mainClass", manifest.getMainClass() );
	}
	
	/**
	 * Initializes and loads settings.
	 */
	private void loadSettings() {
		NODE_SETTING = settingsUtils.newNodeSetting( "HELLO_WORLD", null, "Hello World", I_ICON );
		
		GROUP_HELLO_WORLD = settingsUtils.newSettingsGroup( "Hello World" );
		LIKE_SETTING = settingsUtils.newBoolSetting( "LIKE", NODE_SETTING, GROUP_HELLO_WORLD, ISkillLevel.BASIC, "I like this External Module", null,
		        Boolean.FALSE );
		MAX_TABLE_ROWS_SETTING = settingsUtils.newIntSetting( "MAX_TABLE_ROWS", NODE_SETTING, GROUP_HELLO_WORLD, ISkillLevel.BASIC, "Max rows in the Events table", null, 60, 1, 10000 );
		
		final List< ISetting< ? > > validSettingList = Arrays.< ISetting< ? > > asList( LIKE_SETTING, MAX_TABLE_ROWS_SETTING );
		settings = modEnv.initSettingsBean( "hello-world-settings", validSettingList );
	}
	
	/** Store page comp if created. */
	private PageComp pageComp;
	
	/**
	 * Builds the GUI of the external module. This must be called from the EDT.
	 */
	private void buildGui() {
		// Create a new page
		services.getMainFrame().getMultiPageComp().addPage( guiFactory.newPage( "Hello World", I_ICON, false, new IPageCompCreator< JComponent >() {
			@Override
			public JComponent createPageComp() {
				return pageComp = new PageComp( HelloWorld.this );
			}
		} ) );
		services.getMainFrame().getMultiPageComp().rebuildPageTree( false );
	}
	
	/**
	 * Installs a new replay listener.
	 */
	private void installRepListener() {
		// Register a new replay listener
		services.getRepFolderMonitor().addNewRepListener( new INewRepListener() {
			@Override
			public void newRepDetected( final INewRepEvent event ) {
				addEventData( "New replay: " + event.getFile() );
				logger.debug( "This is the " + manifest.getName() + " mod: acknowledging new replay: " + event.getFile() );
				refreshTable();
			}
		} );
	}
	
	/**
	 * Installs a game change listener if supported on the current platform.
	 */
	private void installGameChangeListener() {
		// Do nothing if not supported
		if ( !services.getSc2Monitor().isSupported() )
			return;
		
		services.getSc2Monitor().addGameChangeListener( new GameChangeAdapter() {
			@Override
			public void gameStarted( final IGameChangeEvent event ) {
				addEventData( "Game started" );
				logger.debug( "This is the " + manifest.getName() + " mod: acknowledging game start." );
				refreshTable();
			};
			
			@Override
			public void gameEnded( final IGameChangeEvent event ) {
				addEventData( "Game ended" );
				logger.debug( "This is the " + manifest.getName() + " mod: acknowledging game end." );
				refreshTable();
			}
		} );
	}
	
	/**
	 * Adds a new event data with the specified event text.
	 * 
	 * @param eventText text of the event to add
	 */
	void addEventData( final String eventText ) {
		synchronized ( eventDataList ) {
			// Ensure max table row setting:
			while ( eventDataList.size() >= settings.get( MAX_TABLE_ROWS_SETTING ) )
				eventDataList.remove( 0 );
			eventDataList.add( new EventData( eventText ) );
		}
	}
	
	/**
	 * Refreshes the table of the page component.
	 */
	private void refreshTable() {
		if ( pageComp == null )
			return;
		
		// GUI refresh must be called from the EDT.
		guiUtils.runInEDT( new Runnable() {
			@Override
			public void run() {
				pageComp.rebuildTable();
			}
		} );
	}
	
	@Override
	public void destroy() {
		// Nothing to do
	}
	
}
