/*
 * Project Scelight External Module SDK.
 * 
 * Copyright (c) 2013 Andras Belicza <iczaaa@gmail.com>
 * 
 * See SDK-LICENSE.txt.
 */
package hu.belicza.andras.helloworldextmod;

import hu.scelightapibase.action.IAction;
import hu.scelightapibase.bean.settings.ISettingChangeEvent;
import hu.scelightapibase.bean.settings.ISettingChangeListener;
import hu.scelightapibase.gui.comp.ILabel;
import hu.scelightapibase.gui.comp.IToolBarForTable;
import hu.scelightapibase.gui.comp.multipage.IPageSelectedListener;
import hu.scelightapibase.gui.comp.table.ITable;
import hu.scelightapibase.gui.comp.table.ITableModel;
import hu.scelightapibase.util.IRelativeDate;
import hu.scelightapibase.util.ISkillLevel;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Vector;

import javax.swing.JPanel;
import javax.swing.JToolBar;
import javax.swing.RowSorter.SortKey;
import javax.swing.SortOrder;

/**
 * Page component implementation that is displayed in our own page.
 * 
 * @author Andras Belicza
 */
@SuppressWarnings("serial")
public class PageComp extends JPanel implements IPageSelectedListener {
	
	/** Reference to the Hello World. */
	private final HelloWorld hw;
	
	/** Reference to the event data list. */
	private final List< EventData > eventDataList;
	
	/** Table displaying the events. */
	private final ITable table;
	
	/** Refresh action: rebuilds the table. */
	private final IAction refreshAction;
	
	/** Delete action: removes the selected rows. */
	private final IAction deleteAction;
	
	/** Add action: adds a new custom event. */
	private final IAction addAction;
	
	/** Model index of the Id column. */
	private int idColIdx;
	
	/** Model index of the Time column. */
	private int timeColIdx;
	
	/** Model index of the Ago column. */
	private int agoColIdx;
	
	
	/**
	 * Creates a new {@link PageComp}.
	 * @param hw reference to the Hello World
	 */
	public PageComp( final HelloWorld hw ) {
		super( new BorderLayout() );
		
		this.hw = hw;
		eventDataList = hw.eventDataList;
		
		table = hw.guiFactory.newTable();
		
		refreshAction = hw.guiFactory.newAction( hw.I_ARROW_CIRCLE_315, "Refresh Table", this, new ActionListener() {
			@Override
			public void actionPerformed( final ActionEvent event ) {
				rebuildTable();
			}
		} );
		
		deleteAction = hw.guiFactory.newAction( hw.I_CROSS, "Remove selected events (Delete)", this, new ActionListener() {
			@Override
			public void actionPerformed( final ActionEvent event ) {
				synchronized ( eventDataList ) {
					final int[] rows = table.getSelectedModelRows();
					// Removing elements shifts the remaining elements afterwards. So have to remove going downward.
					hw.utils.sortReversed( rows );
					for ( final int row : rows )
						eventDataList.remove( row );
				}
				rebuildTable();
			}
		} );
		
		addAction = hw.guiFactory.newAction( hw.I_PLUS_BUTTON, "Add a New Custom Event", this, new ActionListener() {
			@Override
			public void actionPerformed( final ActionEvent event ) {
				hw.addEventData( "Custom event" );
				rebuildTable();
			}
		} );
		
		buildGui();
		
		// We implement IPageSelectedListener so table will be refreshed when page is selected, no need to call rebuildTable() here.
	}
	
	/**
	 * Builds the GUI of the page.
	 */
	private void buildGui() {
		final IToolBarForTable toolBar = hw.guiFactory.newToolBarForTable( table );
		final JToolBar jtoolBar = toolBar.asToolBar();
		jtoolBar.add( refreshAction );
		jtoolBar.addSeparator();
		toolBar.addSelectInfoLabel( "Select an event." );
		toolBar.addSelEnabledButton( deleteAction );
		jtoolBar.addSeparator();
		jtoolBar.add( addAction );
		jtoolBar.addSeparator();
		final ILabel likedLabel = hw.guiFactory.newLabel();
		// Setting change listeners are stored as weak references, so we have to store a reference to them to prevent them being "vanished".
		// That is what "bind" ought to do.
		final ISettingChangeListener scl = new ISettingChangeListener() {
			@Override
			public void valuesChanged( final ISettingChangeEvent event ) {
				if ( event.affected( hw.LIKE_SETTING ) ) {
					final boolean liked = event.get( hw.LIKE_SETTING );
					likedLabel.asLabel().setText( liked ? "You like me!" : "Please like me!" );
					likedLabel.asLabel().setIcon( liked ? hw.I_DUMMY_HAPPY.get() : hw.I_DUMMY_SAD.get() );
				}
			}
		};
		hw.settingsGui.addBindExecuteScl( scl, hw.settings, hw.LIKE_SETTING.selfSet(), likedLabel.asLabel() );
		jtoolBar.add( likedLabel.asLabel() );
		jtoolBar.add( hw.settingsGui.createSettingLink( hw.NODE_SETTING ).leftBorder( 10 ).asLabel() );
		jtoolBar.addSeparator();
		jtoolBar.add( hw.guiFactory.newHelpIcon( hw.HELLO_WORLD_HELP ).asLabel() );
		jtoolBar.addSeparator();
		final ILabel advancedLabel = hw.guiFactory.newLabel( "<html>This text is visible only if skill level &ge; Advanced</html>" );
		hw.settingsGui.bindVisibilityToSkillLevel( advancedLabel.asLabel(), ISkillLevel.ADVANCED );
		jtoolBar.add( advancedLabel.asLabel() );
		toolBar.finalizeLayout();
		add( jtoolBar, BorderLayout.NORTH );
		
		// Set column headers
		final ITableModel model = table.getXTableModel();
		final Vector< String > columns = new Vector<>( Arrays.asList( "Id", "Time", "Ago", "Event" ) );
		final List< Class< ? > > columnClasses = Arrays.< Class< ? > >asList( Integer.class, Date.class, IRelativeDate.class, String.class );
		idColIdx = 0;
		timeColIdx = 1;
		agoColIdx = 2;
		table.getXTableRowSorter().setColumnDefaultDescs( true, idColIdx, timeColIdx, agoColIdx );
		model.setColumnIdentifiers( columns );
		model.setColumnClasses( columnClasses );
		// Set default sorting
		table.asTable().getRowSorter().setSortKeys( Arrays.asList( new SortKey( idColIdx, SortOrder.DESCENDING ) ) );
		table.setDeleteAction( deleteAction );
		add( table.createWrapperBox( true, table.createToolBarParams( this ) ), BorderLayout.CENTER );
	}
	
	/**
	 * Rebuilds the table.
	 */
	void rebuildTable() {
		table.saveSelection( idColIdx );
		
		final ITableModel model = table.getXTableModel();
		
		model.getDataVector().clear();
		model.fireTableDataChanged();
		
		synchronized ( eventDataList ) {
			for ( final EventData ed : eventDataList )
				model.addRow( new Object[] { ed.id, ed.time, hw.factory.newRelativeDate( ed.time ), ed.text } );
		}
		
		table.restoreSelection( idColIdx );
	}
	
	@Override
	public void pageSelected() {
		// There might have been new events since last selected, so rebuild the table.
		rebuildTable();
	}
	
}
